<?php

class KB_calc extends KB_class {

	private $core;

	function __construct($core) {

		$this->core = $core;

	}


	// Public


	public function sum() {

		$sum = 0;

		foreach (func_get_args() as $arg) {

			if (is_array($arg))
				$arg = call_user_func_array(array($this, "sum"), $arg);

			$number = $this->intify($arg);
			$sum += $number;

		}

		return $sum;

	}


	public function difference($arg1 = 0, $arg2 = 0) {


		$number1 = $this->intify($arg1);
		$number2 = $this->intify($arg2);

		$numbers = array_values($this->core->array->sort(array($number1, $number2)));

		return $numbers[1] - $numbers[0];

	}


	public function substract($arg1 = 0, $arg2 = 0) {


		$number1 = $this->intify($arg1);
		$number2 = $this->intify($arg2);

		return $number1 - $number2;

	}


	public function product() {

		$product = 1;

		foreach (func_get_args() as $arg) {

			if (is_array($arg))
				$arg = call_user_func_array(array($this, "product"), $arg);

			$number = $this->intify($arg);

			$product *= $number;

		}

		return $product;

	}


	public function divide($arg1 = 0, $arg2 = 0) {

		$number1 = $this->intify($arg1);
		$number2 = $this->intify($arg2);

		if (is_empty($number2))
			return false;

		return $number1/$number2;

	}


	public function months($time_start = 0, $time_end = 0, $float = false){

		// UNIX time
		if(preg_match("/^[0-9]+$/",$time_start))
			$time_start = "@".$time_start;

		if(preg_match("/^[0-9]+$/",$time_end))
			$time_end = "@".$time_end;

		$months = 0;


		while (date("U",strtotime($time_start)) < date("U",strtotime($time_end))) {

			$months++;
			$date = date("Y-m-d",strtotime($time_start));


			$month = $this->intify(preg_replace("/([0-9]+)-([0-9]+)-([0-9]+)/","$2",$date));
			$year = $this->intify(preg_replace("/([0-9]+)-([0-9]+)-([0-9]+)/","$1",$date));

			if ($month === 12)
				$date = preg_replace("/([0-9]+)-([0-9]+)-([0-9]+)/",++$year."-01-$3",$date);

			else
				$date = preg_replace("/([0-9]+)-([0-9]+)-([0-9]+)/","$1-".++$month."-$3",$date);

			$time_start = $date;

		}

		return $months;
	}


	public function intify($string) {

		if (is_empty($string))
			return 0;

		if (is_int($string))
			return $string;

		if (is_float($string))
			return $string;

		if (strval(intval($string)) === $string)
			return intval($string);

		if (strval(floatval($string)) === $string)
			return floatval($string);

		if (preg_match("/^[0+]".strval(intval($string))."$/",$string))
			return intval($string);

		if (preg_match("/^[0+]".strval(floatval($string))."$/",$string))
			return floatval($string);

		if (preg_match("/^".strval(floatval($string))."[,.]?[0]+$/",$string))
			return floatval($string);

		if (strtotime($string))
			return strtotime($string);

		// HTML decode

		$string = html_entity_decode($string);

		// Remove units

		if (preg_match("/^([\d,.-]+)([\s\D]+)$/",$string))
			$string = preg_replace("/^([\d,.-]+)([\s\D]+)$/", "$1", $string);

		// Replace number formating
		if (preg_match("/(\d+)\.(\d+),/",$string))
			$string = preg_replace(array("/\./","/,/"), array("","."), $string);

		if (preg_match("/(\d+),(\d+)\./",$string))
			$string = preg_replace("/,/", "", $string);

		if (preg_match("/(\d+),(\d+)/",$string))
			$string = preg_replace("/,/", ".", $string);

		// Remove trailing zeros
		$string = preg_replace(array("/0+$/","/\.$/"),array("",""),$string);

		if(strval(floatval($string)) === $string)
			return floatval($string);

		return 0;
	}



	// Alias

	public function diff($arg1 = 0, $arg2 = 0){
		return $this->difference($arg1, $arg2);
	}

	public function div($arg1 = 0, $arg2 = 0){
		return $this->divide($arg1, $arg2);
	}

	public function sub($arg1 = 0, $arg2 = 0){
		return $this->substract($arg1, $arg2);
	}
}