<?php

class KB_content extends KB_data {

	private $core;

	public $frontend;


	function __construct($core) {
		parent::__construct($core);
		$this->core = $core;
		$this->frontend = new KB_content_frontend($core, $this);
	}


	public function create($array = []) {

		if (is_indexed($array)) {

			$ID = [];

			foreach($array as $entry) {
				$ID[] = $this->create($entry);
			}
			return $ID;
		}

		$array = array_merge(
			array(
				"is_file"=>0,
				"secure"=>0,
				"name"=>false,
				"meta_keywords"=>"",
				"group_ID"=>NULL,
			),
			$array
		);

		$array = $this->create_filealias($array);

		if (!$array)
			return false;

		$ID = parent::create($array);

		if ($filename = fetch("filename",$array))
			$filename = $this->create_file($filename,$ID);

		if ($filename)
			$this->update(array("filename"=>$filename,"ID"=>$ID));

		return $ID;

	}


	public function update($data, $where = []) {

		// Multiple updates
		if (is_indexed($data)) {
			foreach($data as $entry) {
				$status[] = $this->update($entry, $where);
			}
			return $status;
		}

		$array = $this->create_filealias($data);

		if (!$array) {

			if (array_key_exists("filealias",$data))
				unset($data["filealias"]);

			return parent::update($data, $where);
		}

		if ($filename = fetch("filename",$array))
			$filename = $this->create_file($filename,$array["ID"]);

		if ($filename)
			$array["filename"] = $filename;

		return parent::update($array, $where);

	}


	public function secured($entry) {

		if (is_numeric($entry))
			$entry = $this->entry($entry);

		if (are_equal(key_fetch("secure",$entry), "0"))
			return false;

		if (!$this->core->user->logged_in())
			return true;

		if (in_list(
			$this->core->user->data("group_ID"),
			$this->correlates(key_of("ID", $entry), $this->core->user->group)
			))
			return false;

		return true;

	}


	// Private Functions

	private function create_filealias($array){

		if (is_empty(key_fetch("name",$array)))
			return false;

		if (not_empty(key_fetch("filealias",$array)) && !preg_match("-\.html$-",key_fetch("filealias",$array)))
			$array["filealias"] = db_encode($array["filealias"]).".html";

		if (is_empty(key_fetch("filealias",$array)))
			$array["filealias"] = db_encode($array["name"]).".html";

		return $array;

	}


	private function create_file($file,$ID){

		if (is_empty($file))
			return false;

		if (is_empty($ID))
			return false;


		if (!preg_match("-\.php$-",$file))
			$file = db_encode($file).".php";

		$this->core->filesystem->set_base_dir(CONTENT_FOLDER);

		if ($this->core->filesystem->exists($ID."/".$file))
			return $file;

		$this->core->filesystem->create_file($file,$ID);

		return $file;
	}




	// Backwards Compatibility
	function json($content){
		deprecated();
		return $this->core->content->frontend->json($content);
	}


}


class KB_structure extends KB_data_group {

	public $link_root = "";

	private $core;

	function __construct($core) {
		parent::__construct($core);
		$this->core = $core;
		$this->link_root = preg_match("/kitbooth/", $_SERVER["REQUEST_URI"]) ? "/kitbooth/" : "/";
	}


	public function create($array = []) {

		$array = array_merge(
			array(
				"defaultpath"=>0,
				"is_base_dir"=>0,
				"name"=>false,
				// "position"=>"",
				// "group_ID"=>NULL,
			),
			$array
		);

		$array = $this->create_url_path_name($array);

		if (isset($array["template_ID"]))
			if (is_empty($array["template_ID"]))
				$array["template_ID"] = NULL;

		if (!$array)
			return false;

		$ID = parent::create($array);
		return $ID;

	}


	public function update($array, $where = []){

		$array = $this->create_url_path_name($array);

		if (isset($array["template_ID"]))
			if (is_empty($array["template_ID"]))
				$array["template_ID"] = "NULL";

		if (!$array)
			return false;

		return parent::update($array, $where);

	}


	public function duplicate($search = [], $replace = []) {

		if (is_empty($search))
			return false;

		// Backwards Compatibility

			if (is_string($replace) && not_empty($replace)) {

				deprecated("Replace should be an array when using duplicate()");

				$replace = array(
					"name" => $replace
				);
			}

		$replace = $this->core->array->merge(
			$replace,
			array(
				"defaultpath" => "0",
				"url_path_name" => ""
			)
		);

		return parent::duplicate($search, $replace);

	}


	public function delete ($search = []) {

		// Backwards Compatibility

		if (is_string($search) && not_empty($search)) {

			deprecated("Argument should be an array when using delete()");

			$search = array(
				"ID" => $search
			);
		}

		// TODO: Files



		// Database
		return parent::delete($search);

	}


	public function set_default($ID = "") {

		if (any_empty($ID, is_numeric($ID)))
			return false;

		$structure_select = $this->entry($ID);

		$value = ifttt(are_equal(key_of("defaultpath", $structure_select), 0), 1, 0);

		//Set "default structure" on all structures with same parentID to false
		$structure_select = $this->read(array(
			"parent_ID" => $structure_select["parent_ID"]
		));

		foreach ($structure_select as $structure) {
			$this->update(array(
				"ID" => $structure["ID"],
				"defaultpath" => 0
			));
		}

		return $this->update(array(
			"ID" => $ID,
			"defaultpath" => $value
		));
	}


	public function set_base($ID = "") {

		if (any_empty($ID, is_numeric($ID)))
			return false;

		$structure_select = $this->entry($ID);

		$value = ifttt(are_equal(key_of("is_base_dir", $structure_select), 0), 1, 0);

		return $this->update(array(
			"ID" => $ID,
			"is_base_dir" => $value
		));
	}


	public function domains() {

		$domains = $this->core->config("domains");

		if (is_empty($domains))
			return "";

		if (is_empty($ID = key_of($_SERVER["HTTP_HOST"], $domains)))
			return "";

		$structures = $this->parents($ID, true);

		if (is_empty($structures))
			return "";

		$structures = array_reverse((array) key_fetch("url_path_name", $structures));
		return implode("/",$structures)."/";
	}


	public function lids($lid = "") {

		if (not_empty($LIDs_IDs = $this->config("lids_ids")))
			return $LIDs_IDs;

		$LIDs = explode("/",$this->domains().$lid);

		if (strlen($LIDs[0]) == 0)
			$LIDs = [];

		$LIDs_IDs = [];

		// BEGIN - Vervollständigung des Pfades mit standard (default) Werten

		for ($i = 0; $i < count($LIDs)+1; $i++) {

			if ($i==0 || empty($LIDs_IDs[$i-1]))
				$result = $this->read(["parent_ID"=>"NULL"]);

			else
				$result = $this->read_per("parent_ID",$LIDs_IDs[$i-1]);

			$tempvar = 0;

			foreach ($result as $key => $value) {
				if ($value['url_path_name'] == @$LIDs[$i]){
					$tempvar=1;
					$LIDs_IDs[] .= $value['ID'];
					break;
				}
			}

			if ($tempvar==0){
				$tempvar2 = 0;
				foreach ($result as $key => $value){
					if ($value['defaultpath'] == 1) {
						for($j=count($LIDs)-1;$j>=$i;$j--){
							$LIDs[$j+1] = $LIDs[$j];
						}
						$LIDs[$i]=$value['url_path_name'];
						$LIDs_IDs[] .= $value['ID'];
						$tempvar2 = 1;
					}
				}
				if ($tempvar2 == 0){
					break(1);
				}
			}
		}

		$base_dirs = "";

		foreach ($LIDs_IDs as $ID) {

			$entry = $this->entry($ID);

			if ($entry["is_base_dir"] == 1 && $entry["defaultpath"] == 0)  //&& in_array($entry["url_path_name"], $path2)){ //OLD: && $sql["defaultpath"]==0){
				$base_dirs .= $entry["url_path_name"]."/";

		}

		$this->config("lids", $LIDs);
		$this->config("lids_ids", $LIDs_IDs);
		$this->config("base_dirs", $base_dirs);

		return $LIDs_IDs;

	}


	function get_first_free_name($array) {
		$structures = $this->read_all_sorted();
		//Look first if structure name to clone includes a number to increment

		if (preg_match("/(.*)(\d{1,})/", $array['name'], $matches)) {
			$int = $matches[2];
			$rest = $matches[1];
		}
		//If there’s no number add one
		else {
			$int = 1;
			$rest = $array['name'];
		}
		//Create Array with just names of all structures with given parent_ID
		$structures_names = [];
		$array["parent_ID"] = $array["parent_ID"] == NULL ? 0 : $array["parent_ID"]; //Convert NULL to 0

		foreach ($structures[$array["parent_ID"]] as $key => $value){
			$structures_names[] .= $value["name"];
		}

		while(in_array($rest.$int, $structures_names)) {
			$int++;
		}
		return $rest.$int;
	}

	public function has_child($ID = NULL) {

		deprecated();

		$structures = $this->read_all();
		foreach ($structures as $value)
		{
			if ($value["parent_ID"] == $ID) return true;
		}
		return false;
	}

	function has_content($ID = NULL)
	{
		//$content = $this->core->content_o();
		// $content_fast = $this->core->data_o();
		// $content_fast->db_name = "kb_content";
		// $has_content = $content_fast->read_per("group_ID", $ID,1);
		$has_content = $this->core->content->entry(array("group_ID"=>$ID));

		if ($has_content)
			return true;

		return false;
	}

	function structure_ID_to_path($args) {

		$args = array_multimerge(array(
				'ID' => 'NULL',
				'key' => 'parent_ID',
				'path' => 'short',
				'base_dir' => false,
			),string_to_array($args,'ID')
		);

		$ID		= $args['ID'];
		$key	= $args['key'];

		$array = array_merge(array($ID),$this->read_parents($ID,$key));
		$array = array_reverse($array);

		$path = '';
		$domain = '';
		$domain_id = 'NULL';
		//$kitboothpath = preg_match("/kitbooth/", $_SERVER["REQUEST_URI"]) ? "kitbooth/" : "";
		$kitboothpath = "";
		// if ($array[0] != "NULL") array_shift($array);
		$use_link_root = true;

		foreach($array as $value) {

			$use_link_root = true;
			$structure =	$this->entry($value);

			if (!empty($structure) && ($structure['is_base_dir'] == 1 && $structure['defaultpath'] == 0 && !$args['base_dir']) && $this->core->runs("frontend"))
				$path =	''; //Reset String when Base Dir is found.

			else if (!empty($structure) && ($structure['defaultpath'] == 0 || $args['path'] == 'full') && ($structure['is_base_dir'] == 0 || $this->core->runs("backend") || $args['base_dir'] ))
				$path .= $structure['url_path_name'].'/'; //

			if (!empty($structure)
				&& $this->core->runs(["backend","shell"])
				&& is_array($this->core->config('domains'))
				&& (array_key_exists($_SERVER["HTTP_HOST"],$this->core->config('domains')) && $structure["ID"] == key_of($_SERVER["HTTP_HOST"],$this->core->config('domains'))
					|| in_array($structure['ID'],$this->core->config('domains'))
				)){
					$domain = in_array($structure['ID'],$this->core->config('domains')) ? array_search($structure['ID'],$this->core->config('domains')) : $_SERVER["HTTP_HOST"];
					$domain = 'http://'.$domain;
					$domain_id = $structure['ID'];
					$path =	''; //Reset String when in Backend and Domain Structure is found.
			}
		}
		//Special Case: Domain Structure and User Group Structure collapse
		if (empty($domain)
			&& $this->core->runs(["backend","shell"])
			&& key_of($this->core->user->get_user_data('group_ID'),$this->core->config("user_group_has_structure")) != false
			&& $this->core->config('domains')
			&& in_array(key_of($this->core->user->get_user_data('group_ID'),$this->core->config("user_group_has_structure")),$this->core->config('domains'))
			){
				$domain = array_search(key_of($this->core->user->get_user_data('group_ID'),$this->core->config("user_group_has_structure")),$this->core->config('domains'));
				$domain = 'http://'.$domain;
		}

		$user_group_structure = key_of($this->core->user->get_user_data('group_ID'),$this->core->config("user_group_has_structure"));
		if ($user_group_structure != false && !in_array($user_group_structure,$this->read_children($domain_id))) $use_link_root = false;
		$path = $use_link_root ? $this->link_root.$kitboothpath.$path : '/'.$kitboothpath.$path;
		$path = $domain.$path;
		return $path;
	}

	private function create_url_path_name($array){

		if (is_empty(key_fetch("name",$array)))
			return $array;

		if (not_empty(key_fetch("url_path_name",$array)))
			$array["url_path_name"] = db_encode($array["url_path_name"]);

		if (is_empty(key_fetch("url_path_name",$array)))
			$array["url_path_name"] = db_encode($array["name"]);

		return $array;

	}

}


class KB_blog extends KB_content {


	private $core;


	function __construct($core) {

		parent::__construct($core);
		$this->core = $core;

		$this->config("classes", array(
			"post" => "post",
			"link" => "link",
			"content" => "content",
		));
	}


	public function render() {

		$args = func_get_args();

		$sid		= !empty($args[0][0]) ? $args[0][0] : $this->core->content->frontend->loaded('group_ID');
		$limit	= isset($args[0][1]) ? (int)$args[0][1] : false;
		$order	= isset($args[0][2]) ? $args[0][2] : 'created DESC';
		$children = isset($args[0][3]) ? (bool)$args[0][3] : false;
		$include_self	= isset($args[0][4]) ? (bool)$args[0][4] : false;

		$return = [];

		foreach($this->recent_posts($sid, $limit, $order, $children) as $post) {

			// Exclude page containing the blog to be linked
			if (!empty($post) && !$this->secured($post['ID']) && ($include_self == true ||	$post['ID'] != $this->core->content->frontend->loaded('ID'))) {

				$link = $this->core->structure->structure_ID_to_path($post['group_ID']).'/';

				if ($post['filealias'] != 'index.html')
					$link = $link.$post['filealias'];

				$content = $this->core->html->tag("div", array($post["content"]), array("class"=>key_of("content", $this->core->config("class"))));
				$link = $this->core->html->tag("a", array($post["name"]), array("href"=>$link, "class"=>key_of("link", $this->core->config("class"))));

				$return[] = $this->core->html->tag("div", array($link, $content), array("class"=>key_of("post", $this->core->config("class"))));
			}
		}

		return $this->core->html->render($return);
	}


	private function recent_posts($structure_ID = false, $limit = false, $order = false, $children = false) {

		if ($order)
			$this->order($order);

		$structures = $children ? array_merge(array($structure_ID),$this->core->content->group->read_children($structure_ID)) : $structure_ID;

		$recent_posts = $this->core->content->read(array("group_ID" => $structures));

		if ($limit)
			$recent_posts = array_slice($recent_posts, 0, $limit);

		return $recent_posts;
	}


}


class KB_content_frontend extends KB_class {


	private
	 	$content,
		$core;

	public $loaded = [];


	function __construct($core, $content) {

		$this->core = $core;
		$this->content = $content;
	}


	public function loaded($key = false) {

		if ($key)
			return key_of($key, $this->loaded);

		return $this->loaded;

	}


	public function render($cid = false, $lid = "") {

		if (not_empty($cid) && is_numeric($cid)) {
			$content = $this->content->entry(array("ID"=>$cid));
		}

		else {
			$content = $this->search($cid, $this->content->group->lids($lid));
		}

		//If no content is found get 404 error page
		if (is_empty($content))
			$content = $this->error_code('404', $this->content->group->lids());

		//If content is locked return 401
		else if ($this->content->secured($content) )
			$content = $this->error_code('401', $this->content->group->lids());

		$this->core->template->frontend->keywords(key_of("meta_keywords", $content));
		$this->core->template->frontend->title(either($this->core->content->frontend->kit_tags(key_of("meta_title", $content)), NULL));
		$this->core->template->frontend->updated(key_of("updated", $content));
		$this->core->template->frontend->description(either($this->core->content->frontend->kit_tags(key_of("meta_description", $content)), NULL));
		$this->core->template->frontend->updated(key_of("updated", $content));
		$this->core->template->frontend->head($this->kit_tags(key_of("head", $content)));

		// Deprecated
		$this->loaded = $content;

		$page = $this->page($content);

		// This is a test to not replace paths twice when content is loading additional content
		// The method is therefor removed from method page
		$page = $this->replace_paths($page);


		$this->core->template->frontend->message(array(
			"info" => $this->core->message->get_info(),
			"message" => $this->core->message->get_message(),
			"dev" => $this->core->message->get_dev_messages(),
		));

		return $page;
	}


	public function search($cid = "", $LIDs_IDs = [], $i = false) {

		if ($i === false && not_empty($LIDs_IDs))
			$i = count($LIDs_IDs)-1;

		if (not_empty($LIDs_IDs) && isset($LIDs_IDs[$i])){

			//If no content is given search for default content

			if (is_empty($cid)) {

				$entry = $this->content->group->entry($LIDs_IDs[$i]);

				switch (key_of("default_content",$entry)){

					case "pos_first" :
						$this->content->order(array("position" =>"ASC"));
						$content = $this->content->entry(array("group_ID"=>$LIDs_IDs[$i]));
						break;

					case "pos_last" :
						$this->content->order(array("position" => "DESC"));
						$content = $this->content->entry(array("group_ID"=>$LIDs_IDs[$i]));
						break;

					case "created_first" :
						$this->content->order(array("created" => "ASC"));
						$content = $this->content->entry(array("group_ID"=>$LIDs_IDs[$i]));
						break;

					case "created_last" :
						$this->content->order(array("created" => "DESC"));
						$content = $this->content->entry(array("group_ID"=>$LIDs_IDs[$i]));
						break;

					case "index_file" :
						$content = $this->content->entry(array("group_ID"=>$LIDs_IDs[$i],"filealias"=>"index.html"));
						break;

					default :
						$content = either(
							$this->content->entry(array("group_ID"=>$LIDs_IDs[$i],"filealias"=>"index.html")),
							$this->content->entry(array("group_ID"=>$LIDs_IDs[$i],"position"=>"1"))
						);
				}

			}

			else {
				$content = $this->content->entry(array("group_ID"=>$LIDs_IDs[$i],"filealias"=>$cid));
			}

			if (is_empty($content)) {
				unset($LIDs_IDs[$i]);
				$content = $this->search($cid, $LIDs_IDs, --$i);
			}

		}

		else{
			//If no content is given search for index.html
			if ($cid==false){
				$content = $this->content->entry(array("group_ID"=>"NULL","filealias"=>"index.html"));
			}else{
				$content = $this->content->entry(array("group_ID"=>"NULL","filealias"=>$cid));
			}
		}

		return $content;

	}


	public function error_code($code, $LIDs_IDs) {

		$result_content = $this->search((int)$code.'.html', $LIDs_IDs);

		// if (not_empty($result_content) && !$this->content->secured(key_of("ID", $result_content))) // Disabled to hide error code pages from menus
		if (not_empty($result_content))
			return $result_content;


		$error_codes = [
			400 => ["h2"=>"Bad Request","p"=>["A status code of 400 indicates that the server did not understand the request due to bad syntax."]],
			401 => ["h2"=>["Unauthorized"],"p"=>["A 401 status code indicates that before a resource can be accessed, the client must be authorised by the server."]],
			402 => ["h2"=>["Payment Required"],"p"=>["The 402 status code is not currently in use, being listed as \"reserved for future use\"."]],
			403 => ["h2"=>["Forbidden"],"p"=>["A 403 status code indicates that the client cannot access the requested resource. That might mean that the wrong username and password were sent in the request, or that the permissions on the server do not allow what was being asked."]],
			404 => ["h2"=>["Not Found"],"p"=>["The best known of them all, the 404 status code indicates that the requested resource was not found at the URL given, and the server has no idea how long for."]],
			405 => ["h2"=>["Method Not Allowed"],"p"=>["The best known of them all, the 404 status code indicates that the requested resource was not found at the URL given, and the server has no idea how long for."]],
			406 => ["h2"=>["None Acceptable"]],
			407 => ["h2"=>["Proxy Authentication Required"]],
			408 => ["h2"=>["Request Timeout"]],
			409 => ["h2"=>["Conflict"]],
			410 => ["h2"=>["Gone"]],
			450 => ["h2"=>["Internal Server Error"]],
			501 => ["h2"=>["Not implemented"]],
			502 => ["h2"=>["Bad gateway"]],
			503 => ["h2"=>["Service Unavailable"]],
			504 => ["h2"=>["Gateway Timeout, Zeitspanne beim Zugriff auf einen anderen Server wurde überschritten"]],
		];

		$error = array(
			"ID"			=>	"0",
			"is_file"		=>	0,
			"secure"		=>	0,
			"meta_title"	=>	$code,
			"name"			=>	$code,
			"meta_keywords"	=> "",
			"position"		=> 0,
			"group_ID"		=> NULL,
			"created"		=>	date("c"),
			"updated"		=>	date("c"),
		);

		$error["content"] = $this->core->html->render(array(
			$this->core->html->h1($code),
			key_of($code, $error_codes)
		));

		return $error;
	}


	public function page($content = []) {

		if (is_empty($content))
			return "";

		if (is_numeric($content))
			$content = $this->core->content->entry($content);

		$this->config("current", $content);
		$this->config("loaded", $this->core->array->union($this->config("loaded"),array($content)));

		$page = key_of("content", $content);

		if (are_equal(key_of("is_file", $content), 1)) {

			$page = either(
				$this->core->file->read_php(
					$this->core->folder->path($this->core->path("root"),$this->core->path("content"),$content['ID']).$content['filename']
				),
				$this->core->file->read_php(
					$this->core->folder->path($this->core->path("root"),$this->core->path("content")).$content['filename']
				)
			);

		}

		$page = $this->kit_tags($page);
		$page = $this->blog_tags($page);
		$page = $this->menu_tags($page);

		// This is a test to not replace paths twice when content is loading additional content
		// Replace paths takes place in method render
		// $page = $this->replace_paths($page);

		$page = $this->json($page);
		return $page;

	}


	public function json($content = ""){
		$that = $this;
		$needle	=	array
					(
						'/(\{(?:(?:[^{}]{1,10000})|(?R)){1}\})/',
						'/(\{(?:(?:[^{}]{1,100})|(?R)){1,2}\})/',
					);

		$content = preg_replace_callback($needle,
			function($m) use($that){
				return $that->replace_json($m[0]);

			},$content
		);
		return $content;
	}


	public function replace_json($content = ""){

		$case = preg_match("/\{\"[a-zA-Z0-0-_]+\"[ ]*\:[ ]*\{/",$content);

		if ($case) {

			// $content = preg_replace("/([\n\r])/","",$content);
			// $content = json_decode($content,true);
			$content = $this->core->json->decode($content);

			if (!empty($content)){

				foreach($content as $c_key => $c_value){

					if (($var = $this->core->get_var($c_key))	&& isset($content[$c_key][$var]))
						$content = $content[$c_key][$var];

					else if	(!($var = $this->core->get_var($c_key)) && isset($content[$c_key]["default"]))
						$content = $content[$c_key]["default"];

					else if	(($var = $this->core->get_var($c_key)) && !isset($content[$c_key][$var]) && isset($content[$c_key]["default"]))
						$content = $content[$c_key]["default"];

					else
						$bool = true;
				}
				if (!empty($bool)){ $content = json_encode($content);}
				$content = html_entity_decode($content); //Deescapes &quot;
			}
		}

		else{
			// $c = stripslashes($c);

		}
		return $content;
	}


	public function blog_tags($content = "") {

		if (is_empty($content))
			return $content;

		$blog_c = preg_match_all("/(<!--\s*Blog)([^-|^>]*)(\s*-->)/iU", $content, $blog_matches);

		if (!$blog_c)
			return $content;


		for ($blog_for = 0; $blog_for < $blog_c; $blog_for++) {

			$pattern[] = "/".preg_quote($blog_matches[0][$blog_for], '/')."/";

			$blog_options = [];

			if (preg_match('/\([^\)]*\)/',$blog_matches[0][$blog_for]))
				$blog_options = explode(',',preg_replace('/<!--\s*Blog\(|\)\s*-->/i','',$blog_matches[0][$blog_for]));

			$replacement[] = $this->core->content->frontend->kit_tags(
				$this->core->blog->render($blog_options)
			);
		}

		$content = preg_replace($pattern, $replacement, $content);
		return $content;

	}


	public function kit_tags($content = "", $local_vars = []) {

		if (is_empty($content))
			return $content;

		$kit_matches = [];
		$kit_c = preg_match_all("/(<!--\s*Kit:)(.*)(\s*-->)/isU", $content, $kit_matches);
		$replacement = [];
		$pattern = [];

		for ($kit_for = 0; $kit_for < $kit_c; $kit_for++) {

			$pattern[] = "/".preg_quote($kit_matches[0][$kit_for], '/')."/";

			$kit_name		 = preg_replace('/(<!--\s*kit:)([^(]+)(.+)(-->)/is', '\2',$kit_matches[0][$kit_for]);
			$kit_options = preg_replace('/(<!--\s*kit:)([^(]+)\((.*)(\)\s*-->)/is', '\3',$kit_matches[0][$kit_for]);

			if (not_empty($kit_options)) {

				if ($json = $this->core->json->decode($kit_options))
					$kit_options = $json;

				else
					$kit_options = explode(",", str_replace(")", "", trim($kit_options)));
			}

			if (
					($kit = $this->core->kit->load($kit_name, false, "frontend")) &&
					($kit = $this->core->kit->run($kit, $kit_options, $local_vars))
				) {

				if (preg_match("/(<!--\s*Kit:)(.*)(\s*-->)/isU", $kit) > 0)
					$kit = $this->kit_tags($kit, $local_vars);

				$replacement[] = $kit;
			}

			else {
				$replacement[] = "";
			}
		}

		$content = preg_replace($pattern, $replacement, $content,1);
		return $content;
	}


	public function menu_tags($content = "") {

		if (is_empty($content))
			return $content;

		$menu_matches = [];
		$menu_c = preg_match_all("/(<!--\s*menu)([^-|^>]*)(\s*-->)/iU", $content, $menu_matches);

		for ($menu_for = 0; $menu_for < $menu_c; $menu_for++) {

			$pattern_menu[] = "/".preg_quote($menu_matches[0][$menu_for], '/')."/";

			$menu_options = [];

			if (preg_match('/\([^\)]*\)/',$menu_matches[0][$menu_for])) {

				$menu_options = preg_replace('/<!--\s*Menu\(|\)\s*-->/i','',$menu_matches[0][$menu_for]);

				if (!is_array($this->core->json->decode($menu_options)))
					$menu_options = explode(',',$menu_options);

			}

			/*
			Deprecated??

			if (is_indexed($menu_options))
				foreach ($menu_options as $menu_option_key => $menu_option) {

					if (preg_match("/=/",$menu_option)){
							$menu_option = preg_split("/=/",$menu_option);
							$this->core->menu->$menu_option[0] = filter_var($menu_option[1],FILTER_VALIDATE_BOOLEAN,FILTER_NULL_ON_FAILURE) !== NULL ? filter_var($menu_option[1],FILTER_VALIDATE_BOOLEAN,FILTER_NULL_ON_FAILURE) : $menu_option[1];
							unset($menu_options[$menu_option_key]);
					}
				}

			//Reorder $menu_options
			$menu_options = array_merge((array)$menu_options,[]);
			*/

			/*
			ob_start();
			call_user_func_array(array($this->core->menu,"render"),$menu_options);
			$replacement_menu[] = ob_get_clean();
			*/
			$replacement_menu[] = call_user_func_array([$this->core->menu,"render"],$menu_options);
		}

		if (isset($pattern_menu))
			$content = preg_replace($pattern_menu, $replacement_menu, $content);

		return $content;

	}


	public function content_tags($content = "") {

		if (is_empty($content))
			return $content;


		if (!matches("<!--", $content))
			return $content;

		if (!preg_match("/<!--\s*content\(.+\)\s*-->/i",$content))
			return $content;

		$core = $this->core;

		$content = preg_replace_callback("/<!--\s*content\(([^)]*)\)\s*-->/i", function($matches) use($core){
			$ID = key_of(1, $matches);

			if (any_empty($ID, is_numeric($ID)))
				return $core->content->frontend->render(
					$core->get_var("cid"),
					$core->get_var("lid")
				);

			return $core->content->frontend->render(
				$ID
			);


		},$content);

		return $content;

	}


	public function replace_paths($content = "") {

		//  CONTENT REPLACE FUNCTIONS
		//	Add default structures and base directoriess to all html and php elements containing specific keywords in contents.
		//	Ignore all text outside html tags.
		//	Include relative links to all but anchors and headers, ignore relative links in anchors, headers and forms which include special links like mailto:.
		//	Case insensitive.

		$core = $this->core;

		$replace_callback = array (

			"/(<[^a])([^>]+)/i"
			=> function($matches) use($core) { return $core->content->frontend->replace(0, $matches[1],$matches[2]);},
			"/(<(a|form) )([^>]+)/i"
			=> function($matches) use($core) { return $core->content->frontend->replace(1, $matches[1],$matches[3]);},
			"/(<\b\w+\b )([^>]+)/i"
			=> function($matches) use($core) { return $core->content->frontend->replace(2, $matches[1],$matches[2]);},
			"/(header\(['|\"]?Location:\s?)([^\)|^'|\"]+)/i"
			=> function($matches) use($core) { return $core->content->frontend->replace(3, $matches[1],$matches[2]);},
			"/(<style)([^>]*>)([^<]+)/i"
			=> function($matches) use($core) { return $core->content->frontend->replace(4, $matches[1].$matches[2],$matches[3]);},
		);

		$content = preg_replace_callback_array($replace_callback, $content);
		return $content;

	}


	public function replace($i, $start, $search) {

		// $search =  stripslashes($search);
		$core = $this->core;

		$replace_callback = array(

			0 => array(
				"/((background|href|poster|src|url|name=['|\"]?movie['|\"]? value|name=['|\"]?src['|\"]? value){1}=['|\"])([^'|^\"]*)(['|\"]{1}[^>]*)/Ui"
				=> function ($match) use($core) {return $core->content->frontend->replace_content($match[1], $match[3], $match[4]);},
			),
			1 => array(
				"/((href|action){1}=['|\"])(\/[^'|^\"]*)(['|\"]{1}[^>]*)/Ui"
				=> function ($match) use($core) {return $core->content->frontend->replace_content($match[1], $match[3], $match[4]);},
			),
			2 => array(
				"/((url\(){1}['|\"]?)([^'|^\"^\)]*)(['|\"]?\)[^>]*)/Ui"
				=> function ($match) use($core) {return $core->content->frontend->replace_content($match[1], $match[3], $match[4]);},
			),
			3 => array(
				"/([\/][^'|^\"]*)/Ui"
				=> function ($match) use($core) {return $core->content->frontend->replace_content("", $match[1], "");},
			),
			4 => array(
				"/((src|url)\([|'|\"]?)([^'|^\"|\)]*)(['|\"]?\))/Ui"
				=> function ($match) use($core) {return $core->content->frontend->replace_content($match[1], $match[3], $match[4]);},
			),

		);


		$search = preg_replace_callback_array(
			$replace_callback[$i],
			$search
		);

		return $start.$search;


	}


	public function replace_content($b, $c, $d) {

		$base_dirs = "/".$this->core->folder->path(
			$this->core->path("url"),
			$this->core->content->group->config("base_dirs")
		);

		$idofcontent = $this->core->content->frontend->loaded("ID");

		$c_temp = preg_split("/\?/",$c);
		$c = array_shift($c_temp);
		$query = implode("?",$c_temp);

		if (not_empty($query))
			$query = "?".$query;

		$path = $this->core->folder->path(
			$this->core->folder->part($c)
		);

		if ($path == ".")
			$path = "/";

		// else if (substr($path,0,1) != "." && substr($path,0,1) != "/")
			// $path = "/".$path;

		$file = $this->core->file->name($c);
		$ext = $this->core->file->extension($c);
		// $ext	= pathinfo($c, PATHINFO_EXTENSION);
		// $ext = preg_match('/#/',$ext) ? array_shift(explode("#",$ext)) : $ext;
		$b =  stripslashes($b);
		$d =  stripslashes($d);

		if (substr($c,0,1) == "." || substr($c,0,1) == "#" || preg_match("/http(s){0,1}:\/\//",$c) == 1 || substr($c, 0, 2) == "//") {
			$return = $b.$c.$query.$d;
		}

		else if (substr($c,0,1) == "?") {
			$return = $b.$this->core->session->append_SID($c).$d;
		}

		else if ($file != "" && $ext != "" && $ext != "html") {

			// Enable file links to be different than ID folder names
			$idofcontent_temp = ifttt(strlen($path)>1 , "" , "/".$idofcontent);
			$return = $b."/".$this->core->folder->path($this->core->path("url"),$idofcontent_temp,$path).$this->core->session->append_SID($file.$query).$d;
		}

		else if (isset($c)) {
			$return = $b.$base_dirs.$path.$this->core->session->append_SID($file.$query).$d;
		}

		else {
			$return = $b.$base_dirs.$c.$d;
		}

		return $return;
	}
}