<?php

class KB_Session extends KB_class {

	public $kit;

	private static $core;

	function __construct($core) {
		self::$core = $core;
		$this->kit = new KB_Session_kit($core);
	}

	public $session_ID = false;


	public function start() {

		// Rest

		if (self::$core->runs("frontend") && self::$core->runs("rest"))
			return false;

		$user_agent = key_of("HTTP_USER_AGENT", $_SERVER);

		//Exclude Bots

		if (match_any([
			"bot","crawl","slurp","spider","facebook","IPWorks","com.apple.WebKit.Networking","InternetMeasurement","CensysInspect","PythonRequests"
		],$user_agent))
			return false;

		// API Calls

		if (is_empty($user_agent)) {

			if (session_id())
				session_destroy();

			session_start();
			$this->session_ID = session_id();
			return $this->session_ID;
		}


		//Should we start a Cookie Check?

		if ((self::$core->runs("frontend") || self::$core->runs("backend")) && (self::$core->get("PHPSESSID","get") || is_empty(self::$core->get("PHPSESSID", "cookie")))) {

			// Is it the first call? Start Cookie Check using GET variable
			if (!self::$core->get("PHPSESSID", "get")) {

				if (session_id() == "")
					session_start();

				self::$core->refresh(array("PHPSESSID"=>session_id()));
			}

			// Or the second call?
			else {

				//Cookies enabled?
				if (not_empty(self::$core->get("PHPSESSID", "cookie"))) {
					// Restart to normal
					self::$core->refresh(array("PHPSESSID"=>""));
				}

				//Or not?
				else {

					// Cookies disabled keep using GET variable
					if (session_id() == "") {

						$this->session_ID = self::$core->get("PHPSESSID", "get");

						// Fail check
						if (preg_match('/^[a-z0-9]{32}$/', $this->session_ID)) {
							session_id($this->session_ID);
							session_start();
						}
						else {
							session_start();
							$this->session_ID = session_id();
						}
					}
				}
			}
		}

		// Returning User, Cookies enabled
		else if (self::$core->runs("frontend") || self::$core->runs("backend")) {

			if (session_id() == "") {
				$this->session_ID = self::$core->get("PHPSESSID", "cookie");
				session_id($this->session_ID);
				session_start();

			}
		}

		return $this->session_ID;

	}


	public function append_SID($url) {

		if($this->session_ID && is_empty(self::$core->get("PHPSESSID", "cookie"))) {

			if(!preg_match('/[?]/',$url))
				$url .= "?PHPSESSID=".$this->session_ID;
			else if(!preg_match('/PHPSESSID=/',$url))
				$url .= "&PHPSESSID=".$this->session_ID;
		}
		return $url;
	}


	public function ID ()	{

		if (self::$core->runs("frontend") && self::$core->runs("rest"))
		// if (self::$core->runs("rest"))
			return false;

		if (!self::$core->runs("frontend") && !self::$core->runs("backend"))
			return false;

    if (is_empty(session_id()))
        return false;

		if ($this->session_ID)
			return $this->secure_code($this->session_ID);

		return session_id();

	}


	public function get($key = false) {

		if ($key === false)
			return self::$core->get(true, "session");

		return either(
			self::$core->get($key, "set"),
			self::$core->get($key, "session")
		);

	}


	public function set($key = false, $value = NULL) {

		if (is_empty($key))
			return false;

		if ($value === NULL)
			return $this->unset_key($key);


		$_SESSION[$key] = $value;

		self::$core->set($key, $value);

		return true;

	}


	public function cookie($key = false, $value = NULL, $time = NULL, $path = "/") {

		if (is_empty($key))
			return false;

		if (not_empty($value)) {
			setcookie($key, $value, ifttt($time, kb_date("U",$time), 0), $path);
			return true;
		}

		else if (!is_null($value)) {
			if (!is_null(self::$core->get($key, "cookie")))
				setcookie($key,"", time() - 3600);

			return true;
		}

		else {
			return self::$core->get($key, "cookie");
		}
	}


	public function unset_key($key) {

		if($this->check() && !empty($key) && isset($_SESSION[$key])) {
			unset($_SESSION[$key]);
			return TRUE;
		}
		return FALSE;
	}


	public function check() {
		$return = (!isset($_SESSION) || !count($_SESSION)) ? $this->start() : true;
		return $return;
	}


	private function secure_code($session_ID = false) {

		if (is_empty($session_ID))
			$session_ID = session_id();

		if (SHELL)
			return(md5($session_ID));

		return md5(
			$session_ID.
			key_of("HTTP_USER_AGENT",$_SERVER).
			key_of("HTTP_HOST",$_SERVER).
			// key_of("REMOTE_ADDR",$_SERVER)
			key_of("REMOTE_HOST",$_SERVER)
		);
	}
}

class KB_Session_kit extends KB_Session {


	private $core;

	function __construct($core) {
		// parent::__construct($core);
		$this->core = $core;
	}


	function get($key = false) {

		$key = $this->core->kit->current["unique_name"]."__".$key;
		return parent::get($key);

	}

	function set($key = false, $value = NULL) {

		$key = $this->core->kit->current["unique_name"]."__".$key;
		return parent::set($key,$value);

	}
}
