<?php

class KB_versions extends KB_class_extended {

	private $core, $data;

	function __construct($core) {

		$this->core = $core;

		$this->data = $this->core->data("kb_versions",[
			"table_ID" => "ID",
			"entry_ID" => "ID",
			"entry" => "text",
			"datetime" => "now",
			"remarks",
		]);

	}


	public function entry ($ID = "") {

		if (any_empty($ID, is_numeric($ID)))
			return [];

		$entry = $this->data->entry($ID);

		$entry = $this->core->array->merge(
			$this->core->json->decode(key_of("entry", $entry)),
			[
				"ID" => key_of("entry_ID", $entry)
			]
		);

		return $entry;

	}


	public function entries ($table_ID = "") {

		if (any_empty($table_ID, is_numeric($table_ID)))
			return [];

		$entries = $this->data->read([
			"table_ID" => $table_ID,
		]);

		if (is_empty($entries))
			return [];

		$entries = $this->core->array->sort_by($entries, "datetime", "DESC");

		$entries = $this->core->array->unique($entries, "entry_ID");

		return $entries;

	}


	public function get ($table_ID = "", $entry_ID = "") {

		if (any_empty($table_ID, $entry_ID, is_numeric($table_ID), is_numeric($entry_ID)))
			return [];

		$entries = $this->data->read([
			"table_ID" => $table_ID,
			"entry_ID" => $entry_ID,
		]);

		return either($entries, []);

	}


	public function set ($table_ID = "", $entry_ID = "", $entry = [], $remarks = "") {

		if (any_empty($table_ID, $entry_ID, is_numeric($table_ID), is_numeric($entry_ID)))
			return false;

		return $this->data->create([
			"table_ID" => $table_ID,
			"entry_ID" => $entry_ID,
			"entry" => $this->core->json->encode($entry),
			"remarks" => $remarks,
		]);

	}

}